@@ -668,6 +668,12 @@ nsCanvasRenderingContext2D::DirtyAllStyles()
 void
 nsCanvasRenderingContext2D::DoDrawImageSecurityCheck(nsIURI* aURI, PRBool forceWriteOnly)
 {
+    // Callers should ensure that mCanvasElement is non-null before calling this
+    if (!mCanvasElement) {
+        NS_WARNING("DoDrawImageSecurityCheck called without canvas element!");
+        return;
+    }
+
     if (mCanvasElement->IsWriteOnly())
         return;
 
@@ -737,6 +743,9 @@ nsCanvasRenderingContext2D::ApplyStyle(PRInt32 aWhichStyle)
 
     nsCanvasPattern* pattern = CurrentState().patternStyles[aWhichStyle];
     if (pattern) {
+        if (!mCanvasElement)
+            return;
+
         DoDrawImageSecurityCheck(pattern->GetURI(), pattern->GetForceWriteOnly());
         pattern->Apply(mCairo);
         return;
@@ -1022,7 +1031,8 @@ nsCanvasRenderingContext2D::Render(nsIRenderingContext *rc)
                                                  NULL);
     CGColorSpaceRef rgb = CGColorSpaceCreateDeviceRGB();
     img = CGImageCreate (mWidth, mHeight, 8, 32, mWidth * 4, rgb,
-                         kCGImageAlphaPremultipliedFirst | CG_BITMAP_BYTE_ORDER_FLAG,
+                         (CGImageAlphaInfo)(kCGImageAlphaPremultipliedFirst |
+                                            CG_BITMAP_BYTE_ORDER_FLAG),
                          dataProvider, NULL, false, kCGRenderingIntentDefault);
     CGColorSpaceRelease (rgb);
     CGDataProviderRelease (dataProvider);
@@ -1857,6 +1867,9 @@ nsCanvasRenderingContext2D::DrawImage()
 {
     nsresult rv;
 
+    if (!mCanvasElement)
+        return NS_ERROR_FAILURE;
+
     nsCOMPtr<nsIXPCNativeCallContext> ncc;
     rv = nsContentUtils::XPConnect()->
         GetCurrentNativeCallContext(getter_AddRefs(ncc));
@@ -2141,10 +2154,15 @@ nsCanvasRenderingContext2D::CairoSurfaceFromElement(nsIDOMElement *imgElt,
             // XXX ERRMSG we need to report an error to developers here! (bug 329026)
             return NS_ERROR_NOT_AVAILABLE;
 
+        PRUint32 status;
+        imgRequest->GetImageStatus(&status);
+        if ((status & imgIRequest::STATUS_LOAD_COMPLETE) == 0)
+            return NS_ERROR_NOT_AVAILABLE;
+
         nsCOMPtr<nsIURI> uri;
         rv = imageLoader->GetCurrentURI(uriOut);
         NS_ENSURE_SUCCESS(rv, rv);
-       
+
         *forceWriteOnlyOut = PR_FALSE;
 
         rv = imgRequest->GetImage(getter_AddRefs(imgContainer));
@@ -2962,6 +2980,9 @@ nsCanvasRenderingContext2D::GetImageData()
 {
     nsresult rv;
 
+    if (!mCanvasElement)
+        return NS_ERROR_FAILURE;
+
     if (mCanvasElement->IsWriteOnly()) {
         nsCOMPtr<nsIScriptSecurityManager> ssm =
             do_GetService(NS_SCRIPTSECURITYMANAGER_CONTRACTID);
@@ -3129,6 +3150,9 @@ nsCanvasRenderingContext2D::PutImageData()
     if (!JS_ConvertArguments (ctx, argc, argv, "ojj", &dataObject, &x, &y))
         return NS_ERROR_DOM_SYNTAX_ERR;
 
+    if (!dataObject)
+        return NS_ERROR_DOM_SYNTAX_ERR;
+
     int32 w, h;
     JSObject *dataArray;
     jsval v;
@@ -3202,9 +3226,10 @@ nsCanvasRenderingContext2D::PutImageData()
     if (mImageSurfaceData) {
         int stride = mWidth*4;
         PRUint8 *dest = mImageSurfaceData + stride*y + x*4;
+        PRUint8 *src = imageBuffer.get();
 
-        for (int32 i = 0; i < y; i++) {
-            memcpy(dest, imgPtr + (w*4)*i, w*4);
+        for (int32 i = 0; i < h; i++) {
+            memcpy(dest, src + (w*4)*i, w*4);
             dest += stride;
         }
     } else {
